// Main application TypeScript with PWA support
import { 
    TranscriptionResponse, 
    TranslationResponse, 
    TTSResponse, 
    TTSServerStatus,
    TTSConfigUpdate,
    TTSConfigResponse,
    TranslationRequest,
    TTSRequest,
    PushPublicKeyResponse,
    TranscriptionRecord,
    TranslationRecord,
    ServiceWorkerRegistrationExtended,
    BeforeInstallPromptEvent
} from './types';

document.addEventListener('DOMContentLoaded', function() {
    // Register service worker
    if ('serviceWorker' in navigator) {
        registerServiceWorker();
    }

    // Initialize app
    initApp();

    // Check for PWA installation prompts
    initInstallPrompt();
});

// Service Worker Registration
async function registerServiceWorker(): Promise<void> {
    try {
        const registration = await navigator.serviceWorker.register('/service-worker.js') as ServiceWorkerRegistrationExtended;
        console.log('Service Worker registered with scope:', registration.scope);

        // Setup periodic sync if available
        if ('periodicSync' in registration && registration.periodicSync) {
            // Request permission for background sync
            const status = await navigator.permissions.query({
                name: 'periodic-background-sync' as PermissionName,
            });

            if (status.state === 'granted') {
                try {
                    // Register for background sync to check for updates
                    await registration.periodicSync.register('translation-updates', {
                        minInterval: 24 * 60 * 60 * 1000, // once per day
                    });
                    console.log('Periodic background sync registered');
                } catch (error) {
                    console.error('Periodic background sync could not be registered:', error);
                }
            }
        }

        // Setup push notification if available
        if ('PushManager' in window) {
            setupPushNotifications(registration);
        }
    } catch (error) {
        console.error('Service Worker registration failed:', error);
    }
}

// Initialize the main application
function initApp(): void {
    // DOM elements
    const recordBtn = document.getElementById('recordBtn') as HTMLButtonElement;
    const translateBtn = document.getElementById('translateBtn') as HTMLButtonElement;
    const sourceText = document.getElementById('sourceText') as HTMLDivElement;
    const translatedText = document.getElementById('translatedText') as HTMLDivElement;
    const sourceLanguage = document.getElementById('sourceLanguage') as HTMLSelectElement;
    const targetLanguage = document.getElementById('targetLanguage') as HTMLSelectElement;
    const playSource = document.getElementById('playSource') as HTMLButtonElement;
    const playTranslation = document.getElementById('playTranslation') as HTMLButtonElement;
    const clearSource = document.getElementById('clearSource') as HTMLButtonElement;
    const clearTranslation = document.getElementById('clearTranslation') as HTMLButtonElement;
    const statusIndicator = document.getElementById('statusIndicator') as HTMLParagraphElement;
    const progressContainer = document.getElementById('progressContainer') as HTMLDivElement;
    const progressBar = document.getElementById('progressBar') as HTMLDivElement;
    const audioPlayer = document.getElementById('audioPlayer') as HTMLAudioElement;
    const ttsServerAlert = document.getElementById('ttsServerAlert') as HTMLDivElement;
    const ttsServerMessage = document.getElementById('ttsServerMessage') as HTMLSpanElement;
    const ttsServerUrl = document.getElementById('ttsServerUrl') as HTMLInputElement;
    const ttsApiKey = document.getElementById('ttsApiKey') as HTMLInputElement;
    const updateTtsServer = document.getElementById('updateTtsServer') as HTMLButtonElement;
    const loadingOverlay = document.getElementById('loadingOverlay') as HTMLDivElement;
    const loadingText = document.getElementById('loadingText') as HTMLParagraphElement;

    // Set initial values
    let isRecording: boolean = false;
    let mediaRecorder: MediaRecorder | null = null;
    let audioChunks: Blob[] = [];
    let currentSourceText: string = '';
    let currentTranslationText: string = '';
    let currentTtsServerUrl: string = '';

    // Check TTS server status on page load
    checkTtsServer();

    // Check for saved translations in IndexedDB
    loadSavedTranslations();

    // Update TTS server URL and API key
    updateTtsServer.addEventListener('click', function() {
        const newUrl = ttsServerUrl.value.trim();
        const newApiKey = ttsApiKey.value.trim();
        
        if (!newUrl && !newApiKey) {
            alert('Please provide at least one value to update');
            return;
        }
        
        const updateData: TTSConfigUpdate = {};
        if (newUrl) updateData.server_url = newUrl;
        if (newApiKey) updateData.api_key = newApiKey;
        
        fetch('/update_tts_config', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify(updateData)
        })
        .then(response => response.json() as Promise<TTSConfigResponse>)
        .then(data => {
            if (data.success) {
                statusIndicator.textContent = 'TTS configuration updated';
                // Save URL to localStorage but not the API key for security
                if (newUrl) localStorage.setItem('ttsServerUrl', newUrl);
                // Check TTS server with new configuration
                checkTtsServer();
            } else {
                alert('Failed to update TTS configuration: ' + data.error);
            }
        })
        .catch(error => {
            console.error('Failed to update TTS config:', error);
            alert('Failed to update TTS configuration. See console for details.');
        });
    });

    // Make sure target language is different from source
    if (targetLanguage.options[0].value === sourceLanguage.value) {
        targetLanguage.selectedIndex = 1;
    }

    // Event listeners for language selection
    sourceLanguage.addEventListener('change', function() {
        if (targetLanguage.value === sourceLanguage.value) {
            for (let i = 0; i < targetLanguage.options.length; i++) {
                if (targetLanguage.options[i].value !== sourceLanguage.value) {
                    targetLanguage.selectedIndex = i;
                    break;
                }
            }
        }
    });

    targetLanguage.addEventListener('change', function() {
        if (targetLanguage.value === sourceLanguage.value) {
            for (let i = 0; i < sourceLanguage.options.length; i++) {
                if (sourceLanguage.options[i].value !== targetLanguage.value) {
                    sourceLanguage.selectedIndex = i;
                    break;
                }
            }
        }
    });

    // Record button click event
    recordBtn.addEventListener('click', function() {
        if (isRecording) {
            stopRecording();
        } else {
            startRecording();
        }
    });

    // Function to start recording
    function startRecording(): void {
        // Request audio with specific constraints for better compression
        const audioConstraints = {
            audio: {
                channelCount: 1, // Mono audio (reduces size by 50%)
                sampleRate: 16000, // Lower sample rate for speech (16kHz is enough for speech)
                echoCancellation: true,
                noiseSuppression: true,
                autoGainControl: true
            }
        };
        
        navigator.mediaDevices.getUserMedia(audioConstraints)
            .then(stream => {
                // Use webm/opus for better compression (if supported)
                const mimeType = MediaRecorder.isTypeSupported('audio/webm;codecs=opus') 
                    ? 'audio/webm;codecs=opus' 
                    : 'audio/webm';
                
                const options = {
                    mimeType: mimeType,
                    audioBitsPerSecond: 32000 // Low bitrate for speech (32 kbps)
                };
                
                try {
                    mediaRecorder = new MediaRecorder(stream, options);
                } catch (e) {
                    // Fallback to default if options not supported
                    console.warn('Compression options not supported, using defaults');
                    mediaRecorder = new MediaRecorder(stream);
                }
                
                audioChunks = [];

                mediaRecorder.addEventListener('dataavailable', event => {
                    audioChunks.push(event.data);
                });

                mediaRecorder.addEventListener('stop', async () => {
                    // Create blob with appropriate MIME type
                    const mimeType = mediaRecorder?.mimeType || 'audio/webm';
                    const audioBlob = new Blob(audioChunks, { type: mimeType });
                    
                    // Log compression results
                    const sizeInKB = (audioBlob.size / 1024).toFixed(2);
                    console.log(`Audio compressed to ${sizeInKB} KB (${mimeType})`);
                    
                    // If the audio is still too large, we can compress it further
                    if (audioBlob.size > 500 * 1024) { // If larger than 500KB
                        statusIndicator.textContent = 'Compressing audio...';
                        const compressedBlob = await compressAudioBlob(audioBlob);
                        transcribeAudio(compressedBlob);
                    } else {
                        transcribeAudio(audioBlob);
                    }
                });

                mediaRecorder.start();
                isRecording = true;
                recordBtn.classList.add('recording');
                recordBtn.classList.replace('btn-primary', 'btn-danger');
                recordBtn.innerHTML = '<div class="recording-wave"><span></span><span></span><span></span><span></span><span></span></div>';
                statusIndicator.textContent = 'Recording... Click to stop';
                statusIndicator.classList.add('processing');
            })
            .catch(error => {
                console.error('Error accessing microphone:', error);
                alert('Error accessing microphone. Please make sure you have given permission for microphone access.');
            });
    }

    // Function to stop recording
    function stopRecording(): void {
        if (!mediaRecorder) return;
        
        mediaRecorder.stop();
        isRecording = false;
        recordBtn.classList.remove('recording');
        recordBtn.classList.replace('btn-danger', 'btn-primary');
        recordBtn.innerHTML = '<i class="fas fa-microphone"></i>';
        statusIndicator.textContent = 'Processing audio...';
        statusIndicator.classList.add('processing');
        showLoadingOverlay('Transcribing your speech...');
        
        // Stop all audio tracks
        mediaRecorder.stream.getTracks().forEach(track => track.stop());
    }
    
    // Function to compress audio blob if needed
    async function compressAudioBlob(blob: Blob): Promise<Blob> {
        return new Promise((resolve) => {
            const audioContext = new (window.AudioContext || (window as any).webkitAudioContext)();
            
            const reader = new FileReader();
            reader.onload = async (e) => {
                try {
                    const arrayBuffer = e.target?.result as ArrayBuffer;
                    const audioBuffer = await audioContext.decodeAudioData(arrayBuffer);
                    
                    // Downsample to 16kHz mono
                    const offlineContext = new OfflineAudioContext(1, audioBuffer.duration * 16000, 16000);
                    const source = offlineContext.createBufferSource();
                    source.buffer = audioBuffer;
                    source.connect(offlineContext.destination);
                    source.start();
                    
                    const compressedBuffer = await offlineContext.startRendering();
                    
                    // Convert to WAV format
                    const wavBlob = audioBufferToWav(compressedBuffer);
                    const compressedSizeKB = (wavBlob.size / 1024).toFixed(2);
                    console.log(`Further compressed to ${compressedSizeKB} KB`);
                    
                    resolve(wavBlob);
                } catch (error) {
                    console.error('Compression failed, using original:', error);
                    resolve(blob); // Return original if compression fails
                }
            };
            reader.readAsArrayBuffer(blob);
        });
    }
    
    // Convert AudioBuffer to WAV format
    function audioBufferToWav(buffer: AudioBuffer): Blob {
        const length = buffer.length * buffer.numberOfChannels * 2;
        const arrayBuffer = new ArrayBuffer(44 + length);
        const view = new DataView(arrayBuffer);
        
        // WAV header
        const writeString = (offset: number, string: string) => {
            for (let i = 0; i < string.length; i++) {
                view.setUint8(offset + i, string.charCodeAt(i));
            }
        };
        
        writeString(0, 'RIFF');
        view.setUint32(4, 36 + length, true);
        writeString(8, 'WAVE');
        writeString(12, 'fmt ');
        view.setUint32(16, 16, true);
        view.setUint16(20, 1, true);
        view.setUint16(22, buffer.numberOfChannels, true);
        view.setUint32(24, buffer.sampleRate, true);
        view.setUint32(28, buffer.sampleRate * buffer.numberOfChannels * 2, true);
        view.setUint16(32, buffer.numberOfChannels * 2, true);
        view.setUint16(34, 16, true);
        writeString(36, 'data');
        view.setUint32(40, length, true);
        
        // Convert float samples to 16-bit PCM
        let offset = 44;
        for (let i = 0; i < buffer.length; i++) {
            for (let channel = 0; channel < buffer.numberOfChannels; channel++) {
                const sample = Math.max(-1, Math.min(1, buffer.getChannelData(channel)[i]));
                view.setInt16(offset, sample * 0x7FFF, true);
                offset += 2;
            }
        }
        
        return new Blob([arrayBuffer], { type: 'audio/wav' });
    }

    // Function to transcribe audio
    function transcribeAudio(audioBlob: Blob): void {
        const formData = new FormData();
        formData.append('audio', audioBlob, 'audio.webm'); // Add filename for better server handling
        formData.append('source_lang', sourceLanguage.value);
        
        // Log upload size
        const sizeInKB = (audioBlob.size / 1024).toFixed(2);
        console.log(`Uploading ${sizeInKB} KB of audio data`);

        showProgress();
        
        fetch('/transcribe', {
            method: 'POST',
            body: formData
        })
        .then(response => response.json() as Promise<TranscriptionResponse>)
        .then(data => {
            hideProgress();
            
            if (data.success && data.text) {
                currentSourceText = data.text;
                sourceText.innerHTML = `<p class="fade-in">${data.text}</p>`;
                playSource.disabled = false;
                translateBtn.disabled = false;
                statusIndicator.textContent = 'Transcription complete';
                statusIndicator.classList.remove('processing');
                statusIndicator.classList.add('success');
                setTimeout(() => statusIndicator.classList.remove('success'), 2000);
                
                // Cache the transcription in IndexedDB
                saveToIndexedDB('transcriptions', {
                    text: data.text,
                    language: sourceLanguage.value,
                    timestamp: new Date().toISOString()
                } as TranscriptionRecord);
            } else {
                sourceText.innerHTML = `<p class="text-danger fade-in">Error: ${data.error}</p>`;
                statusIndicator.textContent = 'Transcription failed';
                statusIndicator.classList.remove('processing');
                statusIndicator.classList.add('error');
                setTimeout(() => statusIndicator.classList.remove('error'), 2000);
            }
        })
        .catch(error => {
            hideProgress();
            console.error('Transcription error:', error);
            sourceText.innerHTML = `<p class="text-danger">Failed to transcribe audio. Please try again.</p>`;
            statusIndicator.textContent = 'Transcription failed';
        });
    }

    // Translate button click event
    translateBtn.addEventListener('click', function() {
        if (!currentSourceText) {
            return;
        }

        statusIndicator.textContent = 'Translating...';
        statusIndicator.classList.add('processing');
        showProgress();
        showLoadingOverlay('Translating to ' + targetLanguage.value + '...');
        
        const requestBody: TranslationRequest = {
            text: currentSourceText,
            source_lang: sourceLanguage.value,
            target_lang: targetLanguage.value
        };
        
        fetch('/translate', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify(requestBody)
        })
        .then(response => response.json() as Promise<TranslationResponse>)
        .then(data => {
            hideProgress();
            
            if (data.success && data.translation) {
                currentTranslationText = data.translation;
                translatedText.innerHTML = `<p class="fade-in">${data.translation}</p>`;
                playTranslation.disabled = false;
                statusIndicator.textContent = 'Translation complete';
                statusIndicator.classList.remove('processing');
                statusIndicator.classList.add('success');
                setTimeout(() => statusIndicator.classList.remove('success'), 2000);
                
                // Cache the translation in IndexedDB
                saveToIndexedDB('translations', {
                    sourceText: currentSourceText,
                    sourceLanguage: sourceLanguage.value,
                    targetText: data.translation,
                    targetLanguage: targetLanguage.value,
                    timestamp: new Date().toISOString()
                } as TranslationRecord);
            } else {
                translatedText.innerHTML = `<p class="text-danger">Error: ${data.error}</p>`;
                statusIndicator.textContent = 'Translation failed';
            }
        })
        .catch(error => {
            hideProgress();
            console.error('Translation error:', error);
            translatedText.innerHTML = `<p class="text-danger">Failed to translate. Please try again.</p>`;
            statusIndicator.textContent = 'Translation failed';
        });
    });

    // Play source text
    playSource.addEventListener('click', function() {
        if (!currentSourceText) return;
        
        playAudio(currentSourceText, sourceLanguage.value);
        statusIndicator.textContent = 'Playing source audio...';
    });

    // Play translation
    playTranslation.addEventListener('click', function() {
        if (!currentTranslationText) return;
        
        playAudio(currentTranslationText, targetLanguage.value);
        statusIndicator.textContent = 'Playing translation audio...';
    });

    // Function to play audio via TTS
    function playAudio(text: string, language: string): void {
        showProgress();
        showLoadingOverlay('Generating audio...');
        
        const requestBody: TTSRequest = {
            text: text,
            language: language
        };
        
        fetch('/speak', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify(requestBody)
        })
        .then(response => response.json() as Promise<TTSResponse>)
        .then(data => {
            hideProgress();
            
            if (data.success && data.audio_url) {
                audioPlayer.src = data.audio_url;
                audioPlayer.onloadeddata = function() {
                    hideLoadingOverlay();
                    // Show audio playing animation
                    const playingAnimation = '<div class="audio-playing"><span></span><span></span><span></span><span></span><span></span></div>';
                    statusIndicator.innerHTML = playingAnimation + ' Playing audio...';
                };
                audioPlayer.onended = function() {
                    statusIndicator.innerHTML = '';
                    statusIndicator.textContent = 'Ready';
                    statusIndicator.classList.remove('processing');
                };
                audioPlayer.play();
            } else {
                statusIndicator.textContent = 'TTS failed';
                
                // Show TTS server alert with error message
                ttsServerAlert.classList.remove('d-none');
                ttsServerAlert.classList.remove('alert-success');
                ttsServerAlert.classList.add('alert-warning');
                ttsServerMessage.textContent = data.error || 'TTS failed';
                
                alert('Failed to play audio: ' + data.error);
                
                // Check TTS server status again
                checkTtsServer();
            }
        })
        .catch(error => {
            hideProgress();
            console.error('TTS error:', error);
            statusIndicator.textContent = 'TTS failed';
            
            // Show TTS server alert
            ttsServerAlert.classList.remove('d-none');
            ttsServerAlert.classList.remove('alert-success');
            ttsServerAlert.classList.add('alert-warning');
            ttsServerMessage.textContent = 'Failed to connect to TTS server';
        });
    }

    // Clear buttons
    clearSource.addEventListener('click', function() {
        sourceText.innerHTML = '<p class="text-muted">Your transcribed text will appear here...</p>';
        currentSourceText = '';
        playSource.disabled = true;
        translateBtn.disabled = true;
    });

    clearTranslation.addEventListener('click', function() {
        translatedText.innerHTML = '<p class="text-muted">Translation will appear here...</p>';
        currentTranslationText = '';
        playTranslation.disabled = true;
    });

    // Function to check TTS server status
    function checkTtsServer(): void {
        fetch('/check_tts_server')
            .then(response => response.json() as Promise<TTSServerStatus>)
            .then(data => {
                currentTtsServerUrl = data.url;
                ttsServerUrl.value = currentTtsServerUrl;
                
                // Load saved API key if available
                const savedApiKey = localStorage.getItem('ttsApiKeySet');
                if (savedApiKey === 'true') {
                    ttsApiKey.placeholder = '••••••• (API key saved)';
                }
                
                if (data.status === 'error' || data.status === 'auth_error') {
                    ttsServerAlert.classList.remove('d-none');
                    ttsServerAlert.classList.remove('alert-success');
                    ttsServerAlert.classList.add('alert-warning');
                    ttsServerMessage.textContent = data.message;
                    
                    if (data.status === 'auth_error') {
                        ttsServerMessage.textContent = 'Authentication error with TTS server. Please check your API key.';
                    }
                } else {
                    ttsServerAlert.classList.remove('d-none');
                    ttsServerAlert.classList.remove('alert-warning');
                    ttsServerAlert.classList.add('alert-success');
                    ttsServerMessage.textContent = 'TTS server is online and ready.';
                    setTimeout(() => {
                        ttsServerAlert.classList.add('d-none');
                    }, 3000);
                }
            })
            .catch(error => {
                console.error('Failed to check TTS server:', error);
                ttsServerAlert.classList.remove('d-none');
                ttsServerAlert.classList.remove('alert-success');
                ttsServerAlert.classList.add('alert-warning');
                ttsServerMessage.textContent = 'Failed to check TTS server status.';
            });
    }

    // Progress indicator functions
    function showProgress(): void {
        progressContainer.classList.remove('d-none');
        let progress = 0;
        const interval = setInterval(() => {
            progress += 5;
            if (progress > 90) {
                clearInterval(interval);
            }
            progressBar.style.width = `${progress}%`;
        }, 100);
        (progressBar as any).dataset.interval = interval.toString();
    }

    function hideProgress(): void {
        const interval = (progressBar as any).dataset.interval;
        if (interval) {
            clearInterval(Number(interval));
        }
        progressBar.style.width = '100%';
        setTimeout(() => {
            progressContainer.classList.add('d-none');
            progressBar.style.width = '0%';
        }, 500);
        hideLoadingOverlay();
    }
    
    function showLoadingOverlay(text: string): void {
        loadingText.textContent = text;
        loadingOverlay.classList.add('active');
    }
    
    function hideLoadingOverlay(): void {
        loadingOverlay.classList.remove('active');
    }
}


// IndexedDB functions for offline data storage
function openIndexedDB(): Promise<IDBDatabase> {
    return new Promise((resolve, reject) => {
        const request = indexedDB.open('VoiceTranslatorDB', 1);
        
        request.onupgradeneeded = (event: IDBVersionChangeEvent) => {
            const db = (event.target as IDBOpenDBRequest).result;
            
            // Create stores for transcriptions and translations
            if (!db.objectStoreNames.contains('transcriptions')) {
                db.createObjectStore('transcriptions', { keyPath: 'timestamp' });
            }
            
            if (!db.objectStoreNames.contains('translations')) {
                db.createObjectStore('translations', { keyPath: 'timestamp' });
            }
        };
        
        request.onsuccess = (event: Event) => {
            resolve((event.target as IDBOpenDBRequest).result);
        };
        
        request.onerror = (event: Event) => {
            reject('IndexedDB error: ' + (event.target as IDBOpenDBRequest).error);
        };
    });
}

function saveToIndexedDB(storeName: string, data: TranscriptionRecord | TranslationRecord): void {
    openIndexedDB().then(db => {
        const transaction = db.transaction([storeName], 'readwrite');
        const store = transaction.objectStore(storeName);
        store.add(data);
    }).catch(error => {
        console.error('Error saving to IndexedDB:', error);
    });
}

function loadSavedTranslations(): void {
    openIndexedDB().then(db => {
        const transaction = db.transaction(['translations'], 'readonly');
        const store = transaction.objectStore('translations');
        const request = store.getAll();
        
        request.onsuccess = (event: Event) => {
            const translations = (event.target as IDBRequest).result;
            if (translations && translations.length > 0) {
                // Could add a history section or recently used translations
                console.log('Loaded saved translations:', translations.length);
            }
        };
    }).catch(error => {
        console.error('Error loading from IndexedDB:', error);
    });
}

// PWA installation prompt
function initInstallPrompt(): void {
    let deferredPrompt: BeforeInstallPromptEvent | null = null;
    const installButton = document.createElement('button');
    installButton.style.display = 'none';
    installButton.classList.add('btn', 'btn-success', 'fixed-bottom', 'm-3');
    installButton.innerHTML = 'Install Voice Translator <i class="fas fa-download ml-2"></i>';
    document.body.appendChild(installButton);

    window.addEventListener('beforeinstallprompt', (e: Event) => {
        // Prevent Chrome 67 and earlier from automatically showing the prompt
        e.preventDefault();
        // Stash the event so it can be triggered later
        deferredPrompt = e as BeforeInstallPromptEvent;
        // Update UI to notify the user they can add to home screen
        installButton.style.display = 'block';

        installButton.addEventListener('click', () => {
            // Hide our user interface that shows our install button
            installButton.style.display = 'none';
            // Show the prompt
            if (deferredPrompt) {
                deferredPrompt.prompt();
                // Wait for the user to respond to the prompt
                deferredPrompt.userChoice.then((choiceResult) => {
                    if (choiceResult.outcome === 'accepted') {
                        console.log('User accepted the install prompt');
                    } else {
                        console.log('User dismissed the install prompt');
                    }
                    deferredPrompt = null;
                });
            }
        });
    });
}

// Push notification setup
function setupPushNotifications(swRegistration: ServiceWorkerRegistration): void {
    // Initialize notification UI
    initNotificationUI(swRegistration);
    
    // Check saved preference
    const notificationsEnabled = localStorage.getItem('notificationsEnabled');
    
    if (notificationsEnabled === 'true' && Notification.permission === 'granted') {
        subscribeToPushManager(swRegistration);
    }
}

function initNotificationUI(swRegistration: ServiceWorkerRegistration): void {
    const notificationPrompt = document.getElementById('notificationPrompt') as HTMLDivElement;
    const enableNotificationsBtn = document.getElementById('enableNotifications') as HTMLButtonElement;
    const notificationToggle = document.getElementById('notificationToggle') as HTMLInputElement;
    const saveSettingsBtn = document.getElementById('saveSettings') as HTMLButtonElement;
    
    // Check if we should show the prompt
    const notificationsDismissed = localStorage.getItem('notificationsDismissed');
    const notificationsEnabled = localStorage.getItem('notificationsEnabled');
    
    if (!notificationsDismissed && !notificationsEnabled && Notification.permission === 'default') {
        // Show toast after 5 seconds
        setTimeout(() => {
            const toast = new (window as any).bootstrap.Toast(notificationPrompt);
            toast.show();
        }, 5000);
    }
    
    // Update toggle state
    notificationToggle.checked = notificationsEnabled === 'true';
    
    // Enable notifications button
    enableNotificationsBtn?.addEventListener('click', async () => {
        const permission = await Notification.requestPermission();
        if (permission === 'granted') {
            localStorage.setItem('notificationsEnabled', 'true');
            notificationToggle.checked = true;
            await subscribeToPushManager(swRegistration);
            const toast = new (window as any).bootstrap.Toast(notificationPrompt);
            toast.hide();
            // Simple alert for mobile compatibility
            setTimeout(() => {
                alert('Notifications enabled successfully!');
            }, 100);
        }
    });
    
    // Notification toggle
    notificationToggle?.addEventListener('change', async () => {
        if (notificationToggle.checked) {
            if (Notification.permission === 'default') {
                const permission = await Notification.requestPermission();
                if (permission !== 'granted') {
                    notificationToggle.checked = false;
                    return;
                }
            }
            localStorage.setItem('notificationsEnabled', 'true');
            await subscribeToPushManager(swRegistration);
        } else {
            localStorage.setItem('notificationsEnabled', 'false');
            await unsubscribeFromPushManager(swRegistration);
        }
    });
    
    // Save settings
    saveSettingsBtn?.addEventListener('click', () => {
        const notifyTranscription = (document.getElementById('notifyTranscription') as HTMLInputElement).checked;
        const notifyTranslation = (document.getElementById('notifyTranslation') as HTMLInputElement).checked;
        const notifyErrors = (document.getElementById('notifyErrors') as HTMLInputElement).checked;
        
        localStorage.setItem('notifyTranscription', notifyTranscription.toString());
        localStorage.setItem('notifyTranslation', notifyTranslation.toString());
        localStorage.setItem('notifyErrors', notifyErrors.toString());
        
        // Show inline success message
        const saveStatus = document.getElementById('settingsSaveStatus') as HTMLDivElement;
        if (saveStatus) {
            saveStatus.style.display = 'block';
            
            // Hide after 2 seconds and close modal
            setTimeout(() => {
                saveStatus.style.display = 'none';
                const modal = (window as any).bootstrap.Modal.getInstance(document.getElementById('settingsModal'));
                modal.hide();
            }, 1500);
        }
    });
    
    // Load saved preferences
    const notifyTranscription = document.getElementById('notifyTranscription') as HTMLInputElement;
    const notifyTranslation = document.getElementById('notifyTranslation') as HTMLInputElement;
    const notifyErrors = document.getElementById('notifyErrors') as HTMLInputElement;
    
    notifyTranscription.checked = localStorage.getItem('notifyTranscription') !== 'false';
    notifyTranslation.checked = localStorage.getItem('notifyTranslation') !== 'false';
    notifyErrors.checked = localStorage.getItem('notifyErrors') === 'true';
}

async function subscribeToPushManager(swRegistration: ServiceWorkerRegistration): Promise<void> {
    try {
        // Get the server's public key
        const response = await fetch('/api/push-public-key');
        const data: PushPublicKeyResponse = await response.json();
        
        // Convert the base64 string to Uint8Array
        function urlBase64ToUint8Array(base64String: string): Uint8Array {
            const padding = '='.repeat((4 - base64String.length % 4) % 4);
            const base64 = (base64String + padding)
                .replace(/-/g, '+')
                .replace(/_/g, '/');
            
            const rawData = window.atob(base64);
            const outputArray = new Uint8Array(rawData.length);
            
            for (let i = 0; i < rawData.length; ++i) {
                outputArray[i] = rawData.charCodeAt(i);
            }
            return outputArray;
        }
        
        const convertedVapidKey = urlBase64ToUint8Array(data.publicKey);
        
        // Subscribe to push notifications
        const subscription = await swRegistration.pushManager.subscribe({
            userVisibleOnly: true,
            applicationServerKey: convertedVapidKey
        });
        
        // Send the subscription details to the server
        await fetch('/api/push-subscribe', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(subscription)
        });
        
        console.log('User is subscribed to push notifications');
    } catch (error) {
        console.error('Failed to subscribe to push notifications:', error);
    }
}

async function unsubscribeFromPushManager(swRegistration: ServiceWorkerRegistration): Promise<void> {
    try {
        const subscription = await swRegistration.pushManager.getSubscription();
        if (subscription) {
            // Unsubscribe from server
            await fetch('/api/push-unsubscribe', {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                },
                body: JSON.stringify(subscription)
            });
            
            // Unsubscribe locally
            await subscription.unsubscribe();
            console.log('User is unsubscribed from push notifications');
        }
    } catch (error) {
        console.error('Failed to unsubscribe from push notifications:', error);
    }
}